<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kbruch" version="1">
<MenuBar>
  <Menu noMerge="0" name="file">
    <text>&amp;File</text>
    <Action name="NewTaskMenu"/>
    <Action name="BackMenu"/>
    <Merge/>
    <Separator/>
    <Action name="file_quit"/>
  </Menu>
</MenuBar>
<ToolBar name="exerciseToolBar" fullWidth="true" lock="true">
  <Action name="NewTaskTool"/>
  <Action name="BackTool"/>  
  <Separator/>
  <Action name="Arithmetic"/>
  <Action name="Comparison"/>
  <Action name="Conversion"/>
  <Action name="MixedNumbers"/>
  <Action name="Factorization"/>
  <Action name="Percentage"/>
</ToolBar>
</kpartgui>
